// Stats

// General
const groups = {
    attackers: newGroup(),
    derattackers: newGroup(),
    defenders: newGroup(),
    sensors: newGroup(),
    repairs: newGroup(),
    spamborgs: newGroup(),
    vattackers: newGroup()
};

// Plays differently if there is at least one sea enemy
var pos;
function needHover_() {
    pos = startPositions;
    var i;
    for (i = 0; i < maxPlayers; i++) {
        if (!propulsionCanReach("wheeled01", pos[me].x, pos[me].y, pos[i].x, pos[i].y)
            && !allianceExistsBetween(me, i)) {
                return true;
        }
    }
    return false;
}

const needHover = needHover_();

// Difficulty related settings
const timerMultiplier = 2 - (difficulty / 2);
switch(difficulty) {
    case EASY:
        var minSensors = 4;
        var minDefDroids = 8;
        break;
    case MEDIUM:
        minSensors = 3;
        minDefDroids = 9;
        break;
    case HARD:
        minSensors = 2;
        minDefDroids = 10;
        break;
    case INSANE:
        minSensors = 2;
        minDefDroids = 8;
        break;
    default:
        debug_("Invalid difficulty");
        break;
}

// Design

// This two functions are preserved for history reasons.
function analyzeWeaponValue(dmg, rate, accur, range) {
    return (dmg - 5) * accur * (rate * 1.2) / (20 - range);
}

var weap;
var bod;
var prop;
var hp;
var value;
function analyzeTemplateValue(template) {
    function searchLi(li, value) {
        var i;
        for (i = 0; i < li.length; i++) {
            if (li[i][0] == value) {
                return li[i];
            }
        }
    }
    weap = searchLi(weapLi, template.weapons[0]);
    prop = searchLi(propLi, template.propulsion);
    bod = searchLi(bodyLi, template.body);
    hp = bod[2] * prop[1];
    value = hp * weap[2];
    if (weap[3] < 5 || prop[2] < 2) {
        value = value / ((30 - (weap[3] * prop[2])) / 15);
    }
    return value / template.cost;
}

var templateLiAP = []; // [template]
var templateLiAT = []; // [template]
var templateLiVtol = []; // [template]
var bodyLi = [["Body1REC", 1], // [body, weight]
              ["Body4ABT", 1],
              ["Body5REC", 2],
              ["Body2SUP", 1],
              ["Body8MBT", 2],
              ["Body3MBT", 1],
              ["Body11ABT", 4],
              ["Body6SUPP", 3],
              ["Body12SUP", 3],
              ["Body9REC", 4],
              ["Body7ABT", 2],
              ["Body10MBT", 3],
              ["Body13SUP", 4]];
var propLi = ["wheeled01", "HalfTrack", needHover ? "hover01" : "tracked01"];

// Update the list when new templates can be designed.
var hq;
function updateTemplateList() {
    var i;
    var y;
    var a;
    // AP list
    for (i in researchPath.weaps_AP) { weap = researchPath.weaps_AP[i];
        for (y in propLi) { prop = propLi[y];
            for (a in bodyLi) { bod = bodyLi[a];
                if (weap[1] - bod[1] <= 1
                    && weap[1] - bod[1] >= -1) {
                        temp = makeTemplate(me, "Temp - CldBePrducdIfGdEngh", bod[0], prop, "", weap[0]);
                        if (temp != null) {
                            templateLiAP[templateLiAP.length] = [temp];
                        }
                    }
            }
        }
    }
    // AT list
    for (i in researchPath.weaps_AT) { weap = researchPath.weaps_AT[i];
        for (y in propLi) { prop = propLi[y];
            for (a in bodyLi) { bod = bodyLi[a];
                if (weap[1] - bod[1] <= 1
                    && weap[1] - bod[1] >= -1) {
                        temp = makeTemplate(me, "Temp - CldBePrducdIfGdEngh", bod[0], prop, "", weap[0]);
                        if (temp != null) {
                            templateLiAT[templateLiAT.length] = [temp];
                        }
                    }
                
            }
        }
    }
    // Vtol list
    for (i in researchPath.weaps_Vtol) { weap = researchPath.weaps_Vtol[i];
            for (a in bodyLi) { bod = bodyLi[a];
                if (weap[1] - bod[1] <= 1
                    && weap[1] - bod[1] >= -1) {
                        temp = makeTemplate(me, "Temp - CldBePrducdIfGdEngh", bod[0], "V-Tol", "", weap[0]);
                        if (temp != null) {
                            templateLiVtol[templateLiAT.length] = [temp];
                        }
                    }
                
            }
        }
}


// Research
var path = randomNum(2);
const researchPath = [
    {
        essential: ["R-Defense-Tower01", "R-Vehicle-Prop-Halftracks", "R-Struc-PowerModuleMk1", "R-Struc-Research-Upgrade01", "R-Vehicle-Body05",
                    "R-Struc-RprFac-Upgrade01", "R-Struc-VTOLPad", needHover ? "R-Vehicle-Prop-Hover" : "R-Vehicle-Prop-Tracks", "R-Vehicle-Body11",
                    "R-Struc-Power-Upgrade03a", "R-Vehicle-Body09", "R-Sys-Autorepair-General", "R-Struc-VTOLPad-Upgrade06", "R-Vehicle-Body13"],
        weaps: ["R-Defense-Tower01", "R-Wpn-MG3Mk1", "R-Wpn-Rocket02-MRL", "R-Wpn-Rocket-ROF03",
                "R-Wpn-Rocket03-HvAT", "R-Wpn-MG5", "R-Wpn-MG-Damage08", "R-Wpn-HvArtMissile"],
        weaps_AP: [["MG1Mk1", 0], ["MG2Mk1", 0], ["MG3Mk1", 1],
                   ["Rocket-MRL", 1], ["MG4ROTARYMk1", 2],
                   ["MG5TWINROTARY", 3], ["Missile-MdArt", 3], ["Missile-HvyArt", 4]],
        weaps_AT: [["Rocket-Pod", 0], ["Rocket-LtA-T", 1], ["Rocket-HvyA-T", 2], ["Missile-A-T", 2]],
        weaps_Vtol: [["Rocket-VTOL-Pod", 0], ["MG3-VTOL", 0], ["Rocket-VTOL-LtA-T", 1],
                     ["Rocket-VTOL-HvyA-T", 1], ["Rocket-VTOL-BB", 1], ["Missile-VTOL-AT", 1]]
    },
    {
        essential: ["R-Defense-Tower01", "R-Vehicle-Prop-Halftracks", "R-Struc-PowerModuleMk1", "R-Struc-Research-Upgrade01", "R-Vehicle-Body05",
                    "R-Struc-RprFac-Upgrade01", "R-Struc-VTOLPad", needHover ? "R-Vehicle-Prop-Hover" : "R-Wpn-Mortar01Lt", "R-Vehicle-Body11",
                    "R-Struc-Power-Upgrade03a", "R-Vehicle-Body09", "R-Sys-Autorepair-General", "R-Struc-VTOLPad-Upgrade06", "R-Vehicle-Body13"],
        weaps: ["R-Wpn-Flamer-ROF01", "R-Wpn-Cannon4AMk1", "R-Wpn-Flamer-ROF03",
                "R-Wpn-Mortar02Hvy", "R-Wpn-Cannon6TwinAslt", "R-Wpn-Mortar-Incendiary", 
                "R-Wpn-Flamer-Damage09", "R-Wpn-Laser02", "R-Wpn-RailGun02",
                "R-Wpn-HvyLaser", "R-Wpn-Rail-ROF03", "R-Wpn-RailGun03"],
        weaps_AP: [["MG1Mk1", 0], ["Flame1Mk1", 1], ["Mortar1Mk1", 1], ["Mortar2Mk1", 3],
                   ["Flame2", 3], ["Mortar-Incendiary", 2], ["PlasmiteFlamer", 1],
                   ["Laser2PULSEMk1", 3], ["HeavyLaser", 4]],
        weaps_AT: [["Cannon1Mk1", 0], ["Cannon2A-TMk1", 3], ["Cannon4AUTOMk1", 3],
                   ["Cannon5VulcanMk1", 3], ["Cannon375mmMk1", 4], ["Cannon6TwinAslt", 4],
                   ["RailGun1Mk1", 1], ["RailGun2Mk1", 3], ["RailGun3Mk1", 4]],
        weaps_Vtol: [["Cannon1-VTOL", 0], ["Cannon4AUTO-VTOL", 1], ["Cannon5Vulcan-VTOL", 1],
                     ["Laser3BEAM-VTOL", 0], ["RailGun2-VTOL", 1], ["Laser2PULSE-VTOL", 1],
                     ["HeavyLaser-VTOL", 1]]
    }][path];

// Cheat on Insane difficulty to get extra power.
function cheatPower() {
    extraPowerTime(60, me);
}

